#!/bin/bash
#
# ##############################################################
# # Copy current parameter values into a new onconfig.std file #
# #      Output goes to stdout.                                #
# ##############################################################
#
if [[ $# -ne 1 ]]; then
    echo 'Usage: $0 template_config >output.config'
    echo ' '
    echo 'Copy current parameter values into a new onconfig.std file'
    echo '     Output goes to stdout.'
    exit 1
fi

OS=$( uname -o )
if [[ $OS = "GNU/Linux" ]]; then
   AWK=awk
else
   if [[ -x gawk ]]; then
      AWK=gawk
   elif [[ -x nawk ]]; then
      AWK=nawk
   else
      AWK=awk
   fi
fi

$AWK  '
      function bld( str,     tmp, i, a, nf ) {
	  tmp="";
	  nf=split( str, a );
	  i=2;
	  do {
	      tmp=tmp a[i] " ";
	      i++;
	  } while (i <= nf);
	  return tmp;
      }
      BEGIN {
	  ntidx=0;
	  bfidx=0;
	  vpidx=0;
	  dlidx=0;
	  FrstBuf=1;
	  FrstNet=1;
	  FrstVP=1;
	  FrstDL=1;
      }
      FILENAME !~ /onconfig.std/ {
	  #
	  ## Save the values from the template ONCONFIG in arrays
	  ## This must be the first file!
	  #
	  if ($1 !~ /^#/) {
	      if ($1 == "NETTYPE") {
		  # Can have multiple NETTYPE entries
		  nettype[ntidx++] = bld( $0 );
	      } else if ($1 == "VPCLASS" ) {
		  # Can have multiple NETTYPE entries
		  vpclass[vpidx++] = bld( $0 );
	      } else if ($1 == "BUFFERPOOL") {
		  # Can have multiple NETTYPE entries
		  bufferpool[bfidx++] = bld( $0 );
	      } else if ($1 == "PRELOAD_DLL_FILE" ) {
		  # Can have multiple PRELOAD_DLL_FILE entries
		  dll_files[dlidx++] = bld( $0 );
	      } else if (NF >= 2) {
		  # Entry with a value
		  param[$1] = $1;
		  value[$1] = bld( $0 );
	      } else if (NF == 1) {
		  # Entry with no value set
		  param[$1] = $1;
		  value[$1] = "";
	      }
	      wrote[$1] = 0;
	  }
      }
      FILENAME ~ /onconfig.std/ {
	  #
	  ## Read the new onconfig.std and install values
	  #
	  # Just print all of the NETTYPE entries together
	  if ($1 !~ /^#/) {
	      # Not a comment, process
	      if ($1 == "NETTYPE") {
		  # Just print all of the NETTYPE entries together
		  if (FrstNet == 0) { next; }
		  for (nt in nettype) {
		      print $1, nettype[nt];
		  }
		  FrstNet = 0;
	      } else if ($1 == "VPCLASS" ) {
		  # Just print all of the VPCLASS entries together
		  if (FrstVP == 0) { next; }
		  for (vp in vpclass) {
		      print $1, vpclass[vp];
		  }
		  FrstVP = 0;
	      } else if ($1 == "BUFFERPOOL") {
		  # Just print all of the BUFFERPOOL entries together
		  if (FrstBuf == 0) { next; }
		  for (bp in bufferpool) {
		      print $1, bufferpool[bp];
		  }
		  FrstBuf = 0;
	      } else if ($1 == "PRELOAD_DLL_FILE" ) {
		  # Just print all preload dll file paths
		  if (FrstDL == 0) { next; }
		  for (fp in dll_files) {
		      print $1, dll_files[fp];
		  }
		  FrstDL = 0;
	      } else if (param[$1] == $1 && value[$1] == "" && NF >= 2) {
		  # Template did not have a value but the onconfig.std does. 
		  # Install that new default value
		  print $1, bld( $0 );
	      } else if (param[$1] == $1) {
		  # Template had a value for this parameter. Copy it in.
		  # But put if the onconfig.std has a default value, also
		  # copy that as a comment
		  if (NF >= 2 && bld( $0 ) != value[$1]) {
		      printf "    ## Std:  %s\n", $0;
		  }
		  print $1, value[$1];
	      } else {
		  # New parameter in the onconfig.std that was not in the templace.
		  # Install it with the default.
		  print $0;
	      }
	      wrote[$1] = 1;
	  } else {
	      # Just print all comment lines. Many explain new parameters!
	      print $0;
	  }
      }
      END {
	  print "\n    #\n    # Parameters not in onconfig.std:\n    #\n";
	  for (mp in wrote) {
	      if (wrote[mp] == 0) {
		  # A parameter in the template file is not in the onconfig.std
		  # Copy it in anyway!
		  print param[mp], value[mp];


	      }
	  }
      }
      ' $1 $INFORMIXDIR/etc/onconfig.std
