#!/bin/ksh

function usage {
    echo "Usage: ${0##*/} [database] [user] [scale]"
    echo "       ${0##*/} -h"
    echo ""
    echo "    database and/user can be a matches wildcard"
    echo "    If not specified, database defaults to '*'"
    echo "    If not specified, user is not reported"
    echo "    Scale is one of KB, MB, or GB"
    echo "    -h - display usage"
}
typeset -u sf
if [[ $# -eq 3 ]]; then sf=$3;
elif [[ $# -eq 2 ]]; then sf=$2;
elif [[ $# -eq 1 ]]; then sf=$1;
else sf="";
fi

if [[ $# -gt 3 || $1 = "-h" || $1 = "-?" ]]; then
    usage
    exit 1
elif [[ $# -eq 3 ]]; then
    dbs=$1
    usr=$2
    usr_str="AND owner matches 'usr' "
    rest=", owner, "
    GROUP="GROUP BY 1, 2"

    if [[ $sf = "KB" ]]; then
	scale=1024
    elif [[ $sf = "MB" ]]; then
	scale=$(( 1024 * 1024 ))
    elif [[ $sf = "GB" ]]; then
	scale=$(( 1024 * 1024 * 1024 ))
    else
	echo "Invalid scale:"
	usage
	exit 1
    fi
elif [[ $# -eq 2 && ($sf = "KB" || $sf = "MB" || $sf = "GB") ]]; then
    dbs=$1
    usr_str=""
    rest=","
    GROUP="GROUP BY 1"

    if [[ $sf = "KB" ]]; then
	scale=1024
    elif [[ $sf = "MB" ]]; then
	scale=$(( 1024 * 1024 ))
    elif [[ $sf = "GB" ]]; then
	scale=$(( 1024 * 1024 * 1024 ))
    fi
elif [[ $# -eq 2 ]]; then
    dbs=$1
    usr=$2
    usr_str="AND owner matches '$usr' "
    rest=", owner, "
    GROUP="GROUP BY 1, 2"
    scale=1024
elif [[ $# -eq 1 && ($sf = "KB" || $sf = "MB" || $sf = "GB") ]]; then
    dbs='*'
    usr_str=""
    rest=","
    GROUP="GROUP BY 1"

    if [[ $sf = "KB" ]]; then
	scale=1024
    elif [[ $sf = "MB" ]]; then
	scale=$(( 1024 * 1024 ))
    elif [[ $sf = "GB" ]]; then
	scale=$(( 1024 * 1024 * 1024 ))
    fi
elif [[ $# -eq 1 ]]; then
    dbs=$1
    usr_str=""
    rest=","
    GROUP="GROUP BY 1"
    scale=1024
elif [[ $# -eq 0 ]]; then
    dbs='*'
    usr_str=""
    rest=","
    GROUP="GROUP BY 1"
    scale=1024
fi
fil="/tmp/dbs_size.$$"
fil2="/tmp/ovrh_size.$$"
sf=${sf:=kb}

dbaccess -e sysmaster - <<EOF  2>/dev/null 1>/dev/null
unload to $fil
select dbsname $rest
       sum(nptotal * pagesize/${scale})::decimal(12,2) allocated,
       sum(npused * pagesize/${scale})::decimal(12,2) used,
       sum((nptotal - npused) * pagesize/${scale})::decimal(12,2) free
from systabnames st, sysptnhdr sp
where st.partnum = sp.partnum
  and st.dbsname matches "$dbs"
  and st.tabname NOT IN ( 'TBLSpace', 'sbspace_desc', 'chunk_adjunc', 'LO_ud_free', 'LO_hdr_partn' )
  and dbsname NOT IN ( 'system', 'sbspace' )
  $usr_str
$GROUP
ORDER BY 1
;
unload to $fil2
select TRIM( CASE 
       	     WHEN tabname = 'TBLSpace' THEN 'TBLSpaces' 
	     WHEN tabname IN ('sbspace_desc', 'chunk_adjunc', 'LO_ud_free', 'LO_hdr_partn' )
	     	  THEN 'SBSpace Metadata'
	     ELSE dbsname END) || ' (internal)' $rest
       sum(nptotal * pagesize/${scale})::decimal(12,2) allocated,
       sum(npused * pagesize/${scale})::decimal(12,2) used,
       sum((nptotal - npused) * pagesize/${scale})::decimal(12,2) free
from systabnames st, sysptnhdr sp
where st.partnum = sp.partnum
  and st.dbsname matches "$dbs"
  and ( st.tabname IN ( 'TBLSpace', 'sbspace_desc', 'chunk_adjunc', 'LO_ud_free', 'LO_hdr_partn' )
      or dbsname = 'system' )
  $usr_str
$GROUP
ORDER BY 1
;
EOF

awk -F\| -v scale=$sf '
BEGIN {
    print "\nSpace usage by database:";
    talloc = tused = tfree = 0.0;
    row=0;
}

NF==5{
    if (row == 0) {
    	printf "\n%-30s  %-12s    %-12s  %-12s\n", "Database", "Allocated " scale, "Used " scale, "Free " scale;
	printf "%-30s  %-12s  %-12s  %-12s  \n", "----------------------------", "------------", "------------", "------------";
        row=1;
    }
    printf "%-30s  %12.2f  %12.2f  %12.2f\n", $1, $2, $3, $4;
    talloc += $2;
    tused += $3;
    tfree += $4;
}
NF==6{
    if (row == 0) {
	printf "\n%-30s  %-18s  %-12s    %-12s  %-12s\n", "Database", "Owner", "Allocated " scale, "Used " scale, "Free " scale;
	printf "%-30s  %-18s  %-12s  %-12s  %-12s  \n", "----------------------------", "------------------", "------------", "------------", "------------";
        row=1;
    }
    printf "%-30s  %-18s  %12.2f  %12.2f  %12.2f\n", $1, $2, $3, $4, $5;
    talloc += $3;
    tused += $4;
    tfree += $5;
}
END {
    if (NF == 5) {
	printf "%-30s  %-12s  %-12s  %-12s\n", "----------------------------", "------------", "------------", "------------";
    } else {
	printf "%-30s  %-18s  %-12s  %-12s  %-12s  \n", "----------------------------", "------------------", "------------", "------------", "------------";
    }
    printf "%-30s  %12.2f  %12.2f  %12.2f\n\n", "Total", talloc, tused, tfree;
}
' $fil $fil2
rm $fil $fil2
exit 0
