#!/bin/ksh

if [[ $# -ne 1 && $# -ne 2 ]]; then
        echo "Usage $0 database <pagesize> "
        echo ""
        exit 0
fi

database=$1;
typeset -i tail tails_p_pg fullpgs l_waste_r l_waste_r;
export tail tails_p_pg fullpgs l_waste_r l_waste_r;

unlfil=/tmp/ask_waste_avg_len.$$

if [[ $# -ne 2 ]]; then
    if [[ "$(uname -s)" = "AIX" ]]; then
        pgsize="4096 8192 12288 16384"
    else
        pgsize="2048 4096 6144 8192 10240 12288 14336 16384"
    fi
else
        pgsize=$2;
fi

function waste_fix {
        l_avg=$1;
        p_pgsz=$2;

        if [[ p_pgsz -lt 2048 ]]; then
                p_pgsz=$(( p_pgsz * 1024 ));
        fi

        l_bytes_p=$(( p_pgsz - 24 ));
        l_usable_p=$l_bytes_p;
        l_rows_p=$(( l_usable_p / (l_avg + 4) ));
        if [[ $l_rows_p -gt 255 ]]; then l_rows_p=255; fi # Max rows per page!

        if [[ $l_avg -gt $(( p_pgsz - 26 )) ]]; then #Row is bigger than usable space on a page!
                pgs_p_row=$(( l_avg / (p_pgsz - 26) ));
                tail=$(( l_avg - (pgs_p_row * (p_pgsz - 26)) ));
                tails_p_pg=$(( (p_pgsz - 24) / (tail + 4) ));
                l_waste_p=$(( ((p_pgsz - 24) - (tails_p_pg * (tail + 4))) / pgs_p_row ));
                l_waste_r=$(( ((p_pgsz - 24) - (tails_p_pg * (tail + 4))) ));
        else
                l_waste_p=$(( (p_pgsz - 24) - (l_rows_p * (l_avg + 4)) ));
                l_waste_r=$(( l_waste_p / l_rows_p ));
        fi
}
function waste_var {
        l_avg=$1;
        p_pgsz=$2;
        typeset -i l_usable_p

        if [[ p_pgsz -lt 2048 ]]; then
                p_pgsz=$(( p_pgsz * 1024 ));
        fi

        l_bytes_p=$(( p_pgsz - 24 ));
        t_usable_p=$( echo "$l_bytes_p * 0.9" | bc )
        l_usable_p=${t_usable_p%%.*}
        l_rows_p=$(( l_bytes_p / (l_avg + 4) ));
        l_rows_pu=$(( l_usable_p / (l_avg + 4) ));
        if [[ $l_rows_p -gt 255 ]]; then l_rows_p=255; fi # Max rows per page!

        if [[ $l_avg -gt $(( p_pgsz - 26 )) ]]; then #Row is bigger than usable space on a page!
                pgs_p_row=$(( l_avg / (p_pgsz - 26) ));
                tail=$(( l_avg - (pgs_p_row * (p_pgsz - 26)) ));
                tails_p_pg=$(( (p_pgsz - 24) / (tail + 4) ));
                l_waste_p=$(( ((p_pgsz - 24) - (tails_p_pg * (tail + 4))) / pgs_p_row ));
                l_waste_r=$(( ((p_pgsz - 24) - (tails_p_pg * (tail + 4))) ));
        else
		if [[ $l_rows_p -eq 1 ]]; then
                       l_waste_p=$(( (p_pgsz - 24) - (l_rows_p * (l_avg + 4)) ));
		       l_waste_r=$(( l_waste_p / l_rows_p ));
		else
        	       l_waste_p=$(( (p_pgsz - 24) - (l_rows_pu * (l_avg + 4)) ));
		       l_waste_r=$(( l_waste_p / l_rows_pu ));
		fi
        fi
}

dbaccess    $database - 2>/dev/null <<EOF
set lock mode to wait 10;
set isolation dirty read;
create procedure ask_waste_avg_length( p_tabname char(128) )
returning integer as avg_length, boolean as variable, bigint as nrows, int as maxlen
        define l_avg, dec, tot, dlen decimal(10,0);
        define l_colname char(128);
        define l_maxlen, l_len, l_coltype int;
        define l_build, l_query lvarchar(32500);
        define variable boolean;
        define l_nrows bigint;

        let l_build = '';
        let variable="F";
        let l_maxlen = 0;
        foreach
                select colname, collength, mod( coltype, 256 )
                into l_colname, l_len, l_coltype
                from systables t, syscolumns c
                where t.tabid = c.tabid
                  and t.tabname = p_tabname
                if (l_coltype = 10 OR l_coltype = 14) then  -- DATETIME
                        let l_build = l_build || ' + ' || mod( (l_len / 256), 256)/2;
                        let l_maxlen = l_maxlen + mod( (l_len / 256), 256)/2;
                elif (l_coltype in (5, 8)) then -- DECIMAL & MONEY
                        let tot=mod((l_len / 256), 256);
                        let dec=mod( l_len, 256);
                        let dlen=tot + ((mod(dec,2) + 3) / 2);
                        let l_build = l_build || ' + ' || dlen;
                        let l_maxlen = l_maxlen + dlen;
                elif (l_coltype in (13, 16 )) then -- VARCHAR & NVARCHAR
                        let l_build = l_build || ' + NVL( length( ' || trim( l_colname ) || ' ), 0) + 1' ;
                        let variable = "T";
                        let l_maxlen = l_maxlen + l_len + 1;
                elif (l_coltype in (19, 20, 21, 22, 40 )) then -- Other variable length types
                        let l_build = l_build || ' + NVL( length( ' || trim( l_colname ) || '::lvarchar ), 0) + 2';
                        let variable = "T";
                        let l_maxlen = l_maxlen + l_len + 2;
                else
                        let l_build = l_build || ' + ' || mod(l_len, 256);
                        let l_maxlen = l_maxlen + l_len;
                end if
        end foreach;

        prepare cntr from 'select count(*) from '||p_tabname;
        declare c_cntr cursor for cntr;
        open c_cntr;
        fetch c_cntr into l_nrows;
        close c_cntr;

        if (l_nrows = 0) then
                return l_maxlen, variable, l_nrows, l_maxlen;
        else
                let l_query = 'select NVL( ceil( avg( 0' || l_build || ' ) ), 0 ) from ' || p_tabname;

                prepare s_query from l_query;

                declare c_query cursor for s_query;

                open c_query;

                fetch c_query into l_avg;

                close c_query;

                return l_avg, variable, l_nrows, l_maxlen;
        end if;
end procedure;
create procedure ask_waste_avg_length_all()
returning lvarchar(128) as tabname, integer as avg_length, boolean as variable, bigint as nrows, int as maxlen
        define l_tabname lvarchar(128);
        define l_avg decimal(10,0);
        define l_variable boolean;
        define l_nrows bigint;
        define l_maxlen int;

        foreach
        select tabname
        into l_tabname
        from systables
        where tabid >= 100 and tabtype = 'T'
                execute procedure ask_waste_avg_length( l_tabname )
                into l_avg, l_variable, l_nrows, l_maxlen;
                return l_tabname, l_avg, l_variable, l_nrows, l_maxlen with resume;
        end foreach;
end procedure;

create temp table ask_values( tabname lvarchar(128), len int, variable boolean, nrows bigint, maxlen int );
insert into ask_values execute procedure ask_waste_avg_length_all( );

unload to $unlfil delimiter ' '
select *
from ask_values;

drop procedure ask_waste_avg_length( char );
drop procedure ask_waste_avg_length_all( );
EOF

wc -l $unlfil >/tmp/lines.$$ 2>/dev/null
if [[ ! -f $unlfil ]]; then
    echo "No tables found"
    echo ""
   exit 0
fi
read nlines dum </tmp/lines.$$
#rm /tmp/lines.$$

if [[ $nlines -eq 0 ]]; then
    echo "No tables found"
    echo ""
    rm $unlfil 2>/dev/null
    exit 1
fi

cat $unlfil | while [[ $nlines -gt 0 ]]; do
    read tabname row_len variable nrows maxlen;

    if [[ "$variable" = "t" ]]; then
        if [[ $nrows -eq 0 ]]; then
            how="Maximum";
        else
            how="Variable Avg";
        fi
        echo "Table: $tabname $how Row Len: $row_len Num Rows: $nrows  Max Len: $maxlen"
    else
        how="Fixed";
        echo "Table: $tabname $how Row Len: $row_len Num Rows: $nrows"
    fi

    echo
    printf "      %9s      %9s      %9s      %9s      %9s\n" "Page Size" "Row Sz" "Rows/Pg" "Waste/Pg" "Waste/Row"
    for pagesize in $pgsize; do
        tail=0; pgs_p_row=0; l_waste_p=0; l_waste_r=0;

        if [[ "$variable" = "t" ]]; then
                waste_var $row_len $pagesize
        else
                waste_fix $row_len $pagesize
        fi

        printf "      %9d      %9d      %9d      %9d      %9d \n" $pagesize $row_len $l_rows_p $l_waste_p $l_waste_r
        if [[ $row_len -gt $(( pagesize - 2 )) ]]; then
            echo "              $pgs_p_row full pages plus $tail bytes on remainder page"
        fi
    done
    echo " ";
    echo " ";
    nlines=$(( nlines - 1 ))
done
# rm -f $unlfil
